#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>

#ifndef NOLOG
#include <syslog.h>
#endif /* NOLOG */

#define BAUDRATE	 B9600
#define DEVICE		"/dev/AVR00"
#define EVT_POWER_DOWN	"/etc/kuroevtd/powerdown"
#define EVT_POWER_UP	"/etc/kuroevtd/powerup"
#define EVT_POWER_PRESS "/etc/kuroevtd/powerpress"
#define EVT_RESET_DOWN	"/etc/kuroevtd/resetdown"
#define EVT_RESET_UP	"/etc/kuroevtd/resetup"
#define EVT_RESET_PRESS "/etc/kuroevtd/resetpress"
#define EVT_LED_FULL "/var/tmp/FULL"
#define EVT_LED_DIAG "/var/tmp/DIAG"

#define SEC_BUTTON_PRESS 3
#define USEC_POLL_TIME (500 * 1000)

struct termios oldtio;
int fd = 0;
int prevfull = 0;
int prevdiag = 0;

check_led_status(char *chkfile, int *prev, int fd, char *on, char *off)
{
  int now = 0;
  int fd2;

  fd2 = open(chkfile, O_RDONLY);
  if (fd2 >= 0) {
    now = 1;
    close(fd2);
  } else {
    now = 0;
  }
  if (*prev != now) {
    if (now == 1) {
      write(fd, on, 4);
    } else {
      write(fd, off, 4);
    }
    *prev = now;
  }
}

int open_serial(char *device,int *fd){
	struct termios newtio;

	*fd = open(device, O_RDWR | O_NOCTTY );

	if(*fd < 0){
		perror(device);
		return -1;
	}

	tcgetattr(*fd,&oldtio); 

	bzero(&newtio, sizeof(newtio));

	newtio.c_cflag = 40717;
	newtio.c_iflag = 16;
	newtio.c_oflag = 1;
	newtio.c_lflag = 0;

	tcflush(*fd,TCIFLUSH);
	tcsetattr(*fd,TCSANOW,&newtio);

	return 0;
}


int close_serial(int fd){
	close(fd);

	return 0;
}


void handle_sigterm(int sig_no){

	if(fd != 0){
		close_serial(fd);
		fd = 0;
	}

	exit(0);
}


int main(int argc,char *argv[]){
	char buf[255];
	int push_power_flag = 0;
	int push_reset_flag = 0;
	int press_power_flag = 0;
	int press_reset_flag = 0;
	time_t time_idle = time(NULL);

	fd_set readfs;
	struct timeval timeout_poll,timeout_press;
	int res;

	if(daemon(0,0) != 0){
		return -1;
	}

	if(signal(SIGTERM,handle_sigterm) == SIG_ERR){
		return -2;
	}

	if(argc == 2){
		if(open_serial(argv[1],&fd) != 0){
			return -3;
		}
	}else{
		if(open_serial(DEVICE,&fd) != 0){
			return -3;
		}
	}

#ifndef NOLOG
	openlog("kuroevtd",LOG_PID,LOG_USER);			
#endif /* NOLOG */

	while(1){
		timeout_poll.tv_usec = USEC_POLL_TIME;
		timeout_poll.tv_sec = 0;
		FD_ZERO(&readfs);
		FD_SET(fd,&readfs);

		res = select(fd + 1,&readfs,NULL,NULL,&timeout_poll);
		
		check_led_status(EVT_LED_FULL, &prevfull, fd, "WWWW", "VVVV");
		check_led_status(EVT_LED_DIAG, &prevdiag, fd, "gggg", "ffff");

		/* catch input */
		if(res != 0){
			read(fd,buf,255);

			switch(buf[0]){
				/* power button release */
				case 0x20:
					if(press_power_flag == 0){
#ifndef NOLOG
						syslog(LOG_USER | LOG_INFO,"power button up"); 
#endif /* NOLOG */
						system(EVT_POWER_UP);
					}

					press_power_flag = 0;
					push_power_flag = 0;

					break;

				/* power button push */
				case 0x21:
#ifndef NOLOG
					syslog(LOG_USER | LOG_INFO,"power button down"); 
#endif /* NOLOG */
					system(EVT_POWER_DOWN);

					press_power_flag = 0;
					push_power_flag = 1;

					break;

				/* reset button release */
				case 0x22:
					if(press_reset_flag == 0){
#ifndef NOLOG
						syslog(LOG_USER | LOG_INFO,"reset button up"); 
#endif /* NOLOG */
						system(EVT_RESET_UP);
					}
				
					press_reset_flag = 0;
					push_reset_flag = 0;

					break;

				/* reset button push */
				case 0x23:
#ifndef NOLOG
					syslog(LOG_USER | LOG_INFO,"reset button down"); 
#endif /* NOLOG */
					system(EVT_RESET_DOWN);

					press_reset_flag = 0;
					push_reset_flag = 1;

					break;

				default:
#ifndef NOLOG
					syslog(LOG_USER | LOG_INFO,"unknown msg %02x",buf[0]); 
#endif /* NOLOG */
					break;

			}

			time_idle = time(NULL);
		}else{
			/* timeout */
			if(time_idle + SEC_BUTTON_PRESS - (USEC_POLL_TIME / 1000 / 1000) < time(NULL)){
				if(push_power_flag == 1){
#ifndef NOLOG
					syslog(LOG_USER | LOG_INFO,"power button press"); 
#endif /* NOLOG */
					system(EVT_POWER_PRESS);

					push_power_flag = 0;
					press_power_flag = 1;
				}

				if(push_reset_flag == 1){
#ifndef NOLOG
					syslog(LOG_USER | LOG_INFO,"reset button press"); 
#endif /* NOLOG */
					system(EVT_RESET_PRESS);

					push_reset_flag = 0;
					press_reset_flag = 1;
				}
			}
		}
	}

	close_serial(fd);

	return 0;
}

